/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *     "Apache Jetspeed" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" or
 *    "Apache Jetspeed", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jetspeed.modules.actions.portlets;

import org.apache.jetspeed.portal.portlets.VelocityPortlet;

// Turbine stuff
import org.apache.turbine.util.Log;
import org.apache.turbine.util.RunData;
import org.apache.turbine.services.Service;
import org.apache.turbine.services.TurbineServices;
import org.apache.jetspeed.om.security.JetspeedUser;

// Velocity Stuff
import org.apache.velocity.context.Context;

//import org.apache.jetspeed.util.StringUtils;
import org.apache.jetspeed.webservices.finance.stockmarket.StockQuoteService;
import org.apache.jetspeed.webservices.finance.stockmarket.StockQuote;
import org.apache.jetspeed.services.resources.JetspeedResources;

/**
 * This action sets up the template context for retrieving stock quotes.
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 */

public class StockQuoteAction extends VelocityPortletAction
{
    /**
     * Build the maximized state content for this portlet. (Same as normal state).
     *
     * @param portlet The velocity-based portlet that is being built.
     * @param context The velocity context for this request.
     * @param rundata The turbine rundata context for this request.
     */
    protected void buildMaximizedContext( VelocityPortlet portlet,
                                          Context context,
                                          RunData rundata )
    {
        buildNormalContext( portlet, context, rundata);
    }

    /**
     * Build the configure state content for this portlet.
     * TODO: we could configure this portlet with configurable skins, etc..
     *
     * @param portlet The velocity-based portlet that is being built.
     * @param context The velocity context for this request.
     * @param rundata The turbine rundata context for this request.
     */
    protected void buildConfigureContext( VelocityPortlet portlet,
                                          Context context,
                                          RunData rundata )
    {
        buildNormalContext( portlet, context, rundata);

        setTemplate(rundata, "stock-quote-customize");


    }

    private static final String SYMBOLS = "symbols";
    private static final String QUOTES = "quotes";

    /**
     * Build the normal state content for this portlet.
     *
     * @param portlet The velocity-based portlet that is being built.
     * @param context The velocity context for this request.
     * @param rundata The turbine rundata context for this request.
     */

    protected void buildNormalContext( VelocityPortlet portlet,
                                       Context context,
                                       RunData rundata )
    {
        try
        {
            StockQuoteService service =
                (StockQuoteService)TurbineServices.getInstance().getService(StockQuoteService.SERVICE_NAME);
            String symbols = (String)rundata.getUser().getTemp(SYMBOLS);
            if (null == symbols || symbols == "")
            {
                String userName = getCurrentUser(rundata);
                String password = getCurrentPassword(rundata);
                symbols = service.getSymbols( "jetspeed", userName, password );
                rundata.getUser().setTemp(SYMBOLS, symbols);
                context.put(SYMBOLS, symbols);
            }
            else
            {
                context.put(SYMBOLS, symbols);
            }
            String [] symbolArray = org.apache.jetspeed.util.StringUtils.stringToArray(symbols, ",");
            StockQuote[] quotes = service.fullQuotes( symbolArray );
            context.put(QUOTES, quotes);

        }
        catch (Exception e)
        {
           // log the error msg
            Log.error(e);

            rundata.setMessage("Error in Jetspeed Stock Quotes: " + e.toString());
            rundata.setStackTrace(org.apache.turbine.util.StringUtils.stackTrace(e), e);
            rundata.setScreenTemplate(JetspeedResources.getString("template.error","Error"));
        }
    }

    private String getCurrentUser(RunData rundata)
    {
        JetspeedUser user = (JetspeedUser)rundata.getUser();
        String userName;

        if (null == user)
        {
            userName = "anon";
        }
        else
        {
            userName = user.getUserName();
            if (null == userName)
            {
                userName = "anon";
            }
        }
        return userName;
    }

    private String getCurrentPassword(RunData rundata)
    {
        JetspeedUser user = (JetspeedUser)rundata.getUser();
        String password;

        if (null == user)
        {
            password = "anon";
        }
        else
        {
            password = user.getPassword();
            if (null == password)
            {
                password = "anon";
            }
        }
        return password;
    }



    public void doUpdate(RunData data, Context context)
    {
        String newSymbols = data.getParameters().getString(SYMBOLS);
        if (null == newSymbols || newSymbols == "")
        {
            try
            {
                StockQuoteService service =
                    (StockQuoteService)TurbineServices.getInstance().getService(StockQuoteService.SERVICE_NAME);
                String symbols = (String)data.getUser().getTemp(SYMBOLS);
                context.put(SYMBOLS, symbols);
                String [] symbolArray = org.apache.jetspeed.util.StringUtils.stringToArray(symbols, ",");
                StockQuote[] quotes = service.fullQuotes( symbolArray );
                context.put(QUOTES, quotes);
                return;
            }
            catch (Exception e)
            {
               // log the error msg
                Log.error(e);

                data.setMessage("Error in Jetspeed Stock Quotes: " + e.toString());
                data.setStackTrace(org.apache.turbine.util.StringUtils.stackTrace(e), e);
                data.setScreenTemplate(JetspeedResources.getString("template.error","Error"));
            }
            return;
        }

        newSymbols = org.apache.jetspeed.util.StringUtils.removeChars(newSymbols, " ");
        try
        {
            StockQuoteService service =
                (StockQuoteService)TurbineServices.getInstance().getService(StockQuoteService.SERVICE_NAME);

            String userName = getCurrentUser(data);
            String password = getCurrentPassword(data);
            boolean success = service.storeSymbols( "jetspeed", userName, password, newSymbols );
            data.getUser().setTemp(SYMBOLS, newSymbols);
            VelocityPortlet portlet = (VelocityPortlet)context.get("portlet");
            buildNormalContext( portlet, context, data);

        }
        catch (Exception e)
        {
           // log the error msg
            Log.error(e);

            data.setMessage("Error in Jetspeed Stock Quotes: " + e.toString());
            data.setStackTrace(org.apache.turbine.util.StringUtils.stackTrace(e), e);
            data.setScreenTemplate(JetspeedResources.getString("template.error","Error"));
        }
    }
}

